package semirings;

import nenok.va.sr.ICSemiring;
import nenok.va.sr.Optimization;
import nenok.va.sr.Semiring;
import nenok.va.sr.SeparativeSemiring;

/**
 * Implementation of the Boolean semiring. This semiring is a strictly monotonic,
 * totally ordered, idempotent C-semiring. Therefore, it qualifies for modeling 
 * optimization problems and consequently, it extends {@link Optimization} 
 * that contains the appropriate ordering relation.
 * 
 * @author Marc Pouly
 * @version $LastChangedRevision: 573 $<br/>$LastChangedDate: 2008-03-26 14:50:22 +0100 (Mi, 26 Mrz 2008) $
 */

public class Boolean implements ICSemiring, Optimization {
		
	/*
	 * Constants:
	 */
	
	/**
	 * Boolean constant 0.
	 */
	
	public static final Boolean ZERO = new Boolean(0);
	
	/**
	 * Boolean constant 1.
	 */
	
	public static final Boolean ONE = new Boolean(1);
	
	/*
	 * Fields:
	 */
	
	private boolean value;
	
	/**
	 * Constructor:
	 * @param value The value of the semiring element.
	 */
	
	public Boolean(boolean value) {
		this.value = value;
	}
	
	/**
	 * Constructor:
	 * @param i The value (either 0 or 1) of the semiring element. 
	 */
	
	public Boolean(int i) {
		if(i != 0 && i != 1) {
			throw new IllegalArgumentException("Value must be either 0 or 1.");
		}
		this.value = (i == 1);
	}
	
	/**
	 * @see nenok.va.sr.Semiring#add(nenok.va.sr.Semiring)
	 */
	
	public Semiring add(Semiring semiring) {
        if(!(semiring instanceof Boolean)) {
            throw new IllegalArgumentException(semiring+" not instance of class "+Boolean.class+".");
        }
		return new Boolean(((Boolean)semiring).value || value);
	}

	/**
	 * @see nenok.va.sr.Semiring#multiply(nenok.va.sr.Semiring)
	 */
	
	public Semiring multiply(Semiring semiring) {
        if(!(semiring instanceof Boolean)) {
            throw new IllegalArgumentException(semiring+" not instance of class "+Boolean.class+".");
        }
		return new Boolean(((Boolean)semiring).value && value);
	}
	
    /**
     * @see java.lang.Object#equals(java.lang.Object)
     */
    
    public boolean equals(Object o) {
        if(!(o instanceof Boolean)) {
            return false;
        }
        return ((Boolean)o).value == value;
    }
    
    /**
     * @see java.lang.Comparable#compareTo(java.lang.Object)
     */
    
    public int compareTo(Object semiring) {
        if(!(semiring instanceof Boolean)) {
            throw new IllegalArgumentException(semiring+" not instance of class "+Boolean.class+".");
        }
    	return Optimization.Implementor.getInstance().compare((Optimization)semiring, this);
    }
    
    /**
     * @see nenok.va.sr.SeparativeSemiring#inverse()
     */
    
    public SeparativeSemiring inverse() {
    	return ICSemiring.Implementor.getInstance().inverse(this);
    }
    
    /**
     * @see java.lang.Object#toString()
     */
    
    public String toString() {
    	return (value)? "1" : "0";
    }
}
