package semirings;

import java.math.BigInteger;

import nenok.va.sr.Optimization;
import nenok.va.sr.Semiring;
import nenok.va.sr.SeparativeSemiring;

/**
 * Implementation of the Tropical semiring with maximization. This semiring is a 
 * strictly monotonic, totally ordered, regular semiring with idempotent addition. 
 * Therefore, it qualifies for modeling  optimization problems and consequently, it 
 * implements {@link Optimization} that contains the appropriate ordering relation.
 * 
 * @author Marc Pouly
 * @version $LastChangedRevision: 573 $<br/>$LastChangedDate: 2008-03-26 14:50:22 +0100 (Mi, 26 Mrz 2008) $
 */

public class MaxTropical implements SeparativeSemiring, Optimization {
			
	private BigInteger value;
			
    /**
     * Constructor:
     * @param value The value for this semiring element.
     */
    
    public MaxTropical(int value) {
        this.value = BigInteger.valueOf(value);
    }
    
    /**
     * Constructor:
     * @param value The value for this semiring element.
     */    
    
    private MaxTropical(BigInteger value) {
    	this.value = value;
    }
    
    /**
     * @see nenok.va.sr.Semiring#multiply(nenok.va.sr.Semiring)
     */
    
    public Semiring multiply(Semiring semiring) {
        if(!(semiring instanceof MaxTropical)) {
            throw new IllegalArgumentException(semiring+" not instance of class "+MaxTropical.class+".");
        }
        return new MaxTropical(value.add(((MaxTropical)semiring).value));
    }
    
    /**
     * @see nenok.va.sr.Semiring#add(nenok.va.sr.Semiring)
     */
    
    public Semiring add(Semiring semiring) {
        if(!(semiring instanceof MaxTropical)) {
            throw new IllegalArgumentException(semiring+" not instance of class "+MaxTropical.class+".");
        }
        return new MaxTropical(value.max(((MaxTropical)semiring).value));
    }
    
    /**
     * @see nenok.va.sr.SeparativeSemiring#inverse()
     */
    
    public SeparativeSemiring inverse() {
    	return new MaxTropical(value.negate());
    }
    
    /**
     * @see java.lang.Object#equals(java.lang.Object)
     */
    
    public boolean equals(Object o) {
        if(!(o instanceof MaxTropical)) {
            throw new IllegalArgumentException(o+" not instance of class "+MaxTropical.class+".");
        }
        return value.equals(((MaxTropical)o).value);
    }
    
    /**
     * @see java.lang.Comparable#compareTo(java.lang.Object)
     */
    
    public int compareTo(Object semiring) {
        if(!(semiring instanceof MaxTropical)) {
            throw new IllegalArgumentException(semiring+" not instance of class "+MaxTropical.class+".");
        }
    	return Optimization.Implementor.getInstance().compare((Optimization)semiring, this);
    }
        
    /**
     * @see java.lang.Object#toString()
     */
    
    public String toString() {
    	return value.toString();
    }
}