package semirings;

import java.math.BigInteger;

import nenok.va.sr.Optimization;
import nenok.va.sr.Semiring;
import nenok.va.sr.SeparativeSemiring;

/**
 * Implementation of the Tropical semiring with minimization. This semiring is a 
 * strictly monotonic, totally ordered, cancellative (therefore separative) semiring. 
 * Therefore, it qualifies for modeling optimization problems and consequently, it 
 * extends {@link Optimization} that contains the appropriate ordering relation.
 * 
 * @author Marc Pouly
 * @version $LastChangedRevision: 573 $<br/>$LastChangedDate: 2008-03-26 14:50:22 +0100 (Mi, 26 Mrz 2008) $
 */

public class MinTropical implements SeparativeSemiring, Optimization {
			
	private BigInteger value;
		
    /**
     * Constructor:
     * @param value The value for this semiring element.
     */
    
    public MinTropical(int value) {
    	if(value < 0) {
    		throw new IllegalArgumentException("This semiring takes only non-negative values.");
    	}
        this.value = BigInteger.valueOf(value);
    }
    
    /**
     * Constructor:
     * @param value The value for this semiring element.
     */    
    
    private MinTropical(BigInteger value) {
    	this.value = value;
    }
    
    /**
     * @see nenok.va.sr.Semiring#multiply(nenok.va.sr.Semiring)
     */
    
    public Semiring multiply(Semiring semiring) {
        if(!(semiring instanceof MinTropical)) {
            throw new IllegalArgumentException(semiring+" not instance of class "+MinTropical.class+".");
        }
        return new MinTropical(value.add(((MinTropical)semiring).value));
    }
    
    /**
     * @see nenok.va.sr.Semiring#add(nenok.va.sr.Semiring)
     */
    
    public Semiring add(Semiring semiring) {
        if(!(semiring instanceof MinTropical)) {
            throw new IllegalArgumentException(semiring+" not instance of class "+MinTropical.class+".");
        }
        return new MinTropical(value.min(((MinTropical)semiring).value));
    }
    
    /**
     * @see java.lang.Object#equals(java.lang.Object)
     */
    
    public boolean equals(Object o) {
        if(!(o instanceof MinTropical)) {
            return false;
        }
        return value.equals(((MinTropical)o).value);
    }
    
    /**
     * @see java.lang.Comparable#compareTo(java.lang.Object)
     */
    
    public int compareTo(Object semiring) {
        if(!(semiring instanceof MinTropical)) {
            throw new IllegalArgumentException(semiring+" not instance of class "+MinTropical.class+".");
        }
    	return Optimization.Implementor.getInstance().compare((Optimization)semiring, this);
    }
    
    /**
     * @see nenok.va.sr.SeparativeSemiring#inverse()
     */
    
    public SeparativeSemiring inverse() {
    	return new MinTropical(value.negate());
    }
    
    /**
     * @see java.lang.Object#toString()
     */
    
    public String toString() {
        return value.toString();
    }
}
