package ppotentials;

import javax.swing.JPanel;

import nenok.va.FiniteVariable;
import nenok.va.Representor;
import nenok.va.Scalability;
import nenok.va.Separativity;
import nenok.va.sr.SRValuation;
import nenok.va.sr.Semiring;

/**
 * Probability potential are an example of a valuation algebra and this
 * class offers an according implementation. It is based on the semiring
 * framework by extensing the pre-implementation of valuation algebras 
 * induced by separative semirings. Furthermore, probability potentials
 * are scalable and therefore, the according interface {@link Scalability} 
 * is implemented.
 * 
 * @author Marc Pouly
 * @version $LastChangedRevision: 568 $<br/>$LastChangedDate: 2008-03-26 14:42:35 +0100 (Mi, 26 Mrz 2008) $
 */

public class Potential extends SRValuation implements Scalability {
			
	/**
	 * @see nenok.va.sr.SRValuation#create(nenok.va.FiniteVariable[], nenok.va.sr.Semiring[])
	 */
	
	@Override
	public SRValuation create(FiniteVariable[] vars, Semiring[] values) {
		return new Potential(vars, cast(values));
	}
	
	/**
	 * Constructor:
	 * @param var The single variable of this semiring valuation. 
	 * @param values The values assigned to each configuration.
	 */
	
	public Potential(FiniteVariable var, double... values) {
		super(new FiniteVariable[] {var}, convert(values));
	}
	
	/**
	 * Constructor:
	 * @param vars The variables of this semiring valuation. 
	 * @param values The semiring value for each possible configuration.
	 */
	
	private Potential(FiniteVariable[] vars, Arithmetic[] values) {
		super(vars, values);
	}
			
	/**
	 * Constructor:
	 * @param var The variable that is conditioned.
	 * @param conditionals The conditional variables.
	 * @param values The value for each configuration.
	 */
	
	public Potential(FiniteVariable var, FiniteVariable[] conditionals, double... values) {
		super(merge(var, conditionals), convert(values));
	}	
	
	/**
	 * @see nenok.va.Separativity#inverse()
	 */
	
	public Separativity inverse() {
		return Separativity.Implementor.getInstance().inverse(this);
	}
	
	/**
	 * @see nenok.va.Scalability#scale()
	 */
	
	public Scalability scale() {
		return Scalability.Implementor.getInstance().scale(this);
	}
	
	/**
	 * Casts an array of {@link Semiring} into an array of {@link Arithmetic}.
	 * @param values The element array to cast.
	 * @return An array of type {@link Arithmetic}.
	 */
	
	private static Arithmetic[] cast(Semiring[] values) {
		Arithmetic[] result = new Arithmetic[values.length];
		for (int i = 0; i < result.length; i++) {
			result[i] = (Arithmetic)values[i];
		}
		return result;
	}
	
	/**
	 * Converts an array of real numbers into semiring elements of the given type.
	 * @param values The input array of real numbers.
	 * @param type The type of semiring values that are created.
	 * @return A corresponding array of the Arithmetic semiring elements.
	 */
	
	private static Arithmetic[] convert(double[] values) {
		Arithmetic[] elements = new Arithmetic[values.length];
		for (int i = 0; i < values.length; i++) {
			elements[i] = new Arithmetic(values[i]);
		}
		return elements;
	}
	
	/**
	 * Adds a variable at the beginning of an existing variable array.
	 * @param var The variable to add.
	 * @param vars The existing variable array.
	 * @return The resulting new variable array.
	 */
	
	private static FiniteVariable[] merge(FiniteVariable var, FiniteVariable[] vars) {
		FiniteVariable[] result = new FiniteVariable[vars.length+1];
		result[0] = var;
		for (int i = 1; i < result.length; i++) {
			result[i] = vars[i-1];
		}
		return result;
	}
	
    /**
     * Displays the current potential as a chart.
     * @return The current potential as a chart.
     */
    
	@Representor
    public JPanel displayChart() {
        return new Chart(this);
    }
}
