package nenok;

import java.io.Serializable;
import java.rmi.RemoteException;

import nenok.adapt.Labeled;
import nenok.va.Domain;
import nenok.va.Valuation;

/**
 * Knowledgebases allow to collect valuations under an identifies
 * that is referred to as the knowledgebase's name. Subclasses refine 
 * this type to knowledgebases with either local or remote data.
 * Note also that knowledgebases are immutable and serializable.
 * 
 * @author Marc Pouly
 * @version $LastChangedRevision: 550 $<br/>$LastChangedDate: 2008-03-26 14:14:27 +0100 (Mi, 26 Mrz 2008) $
 * @param <T> Type to distinguish between remote and local knowledgebases.
 */

public abstract class Knowledgebase<T extends Labeled> implements Serializable {
	
	private final String name;
		
	/**
	 * Factory method:
	 * Creates a knowledgebase from valuations that are local.
	 * @param vals An array of valuation objects that belong to this knowledgebase.
	 * @param name The identifier of this knowledgebase.
	 * @return A knowledgebase with the given name and content.
	 */
	
	public static Knowledgebase create(Valuation[] vals, String name) {
		return new LocalKB(vals, name);
	}
	
	/**
	 * Constructor:
	 * @param name The identifier of this knowledgebase.
	 */
	
	protected Knowledgebase(String name) {
		this.name = name;
	}

	/**
	 * @return The set of valuations that belong to this knowledgebase.
	 * @throws RemoteException Exception caused by communication problems if valuations
	 * are read from a remote processor.
	 */

	public abstract Valuation[] getValuations() throws RemoteException;
	
	/**
	 * @return The content of this knowledgebase as an array of transmissibles.
	 */
	
	public abstract T[] toArray();
	
	/**
	 * @return The total domain of this knowledgebase.
	 */

	public abstract Domain getDomain();

	/**
	 * @return The type of valuations that is contained in this knowledgebase.
	 * @throws Exception Wrapper exception.
	 */

	public abstract Class getType() throws Exception;
	
	/**
	 * @return The size of this knowledgebase.
	 */

	public abstract int size();
	
	/**
	 * @see java.lang.Object#toString()
	 */
	
	@Override
	public String toString() {
		return name;
	}
}