package nenok;

import java.net.MalformedURLException;
import java.net.UnknownHostException;
import java.rmi.RMISecurityManager;
import java.rmi.RemoteException;
import java.util.Hashtable;
import java.util.logging.Level;
import java.util.logging.Logger;

import nenok.net.processor.Processor;
import nenok.net.processor.Starter;
import nenok.net.registry.Registry;
import net.jini.core.discovery.LookupLocator;
import net.jini.core.entry.Entry;
import net.jini.core.lookup.ServiceItem;
import net.jini.core.lookup.ServiceTemplate;
import net.jini.discovery.DiscoveryGroupManagement;
import net.jini.discovery.LookupDiscoveryManager;
import net.jini.id.Uuid;
import net.jini.id.UuidFactory;
import net.jini.lease.LeaseRenewalManager;
import net.jini.lookup.LookupCache;
import net.jini.lookup.ServiceDiscoveryManager;
import net.jini.lookup.entry.Name;

/**
 * This utility class deals with Nenok services. It provides static methods to start, localize &amp; terminate
 * Nenok {@link Processor}s. Furthermore, localization of {@link Registry} services is an important task of 
 * this class, which is implemented as multicast localization. To assert best possible lookup times a static 
 * JINI LookupCache is used.
 * 
 * @author Marc Pouly
 * @version $LastChangedRevision: 550 $<br/>$LastChangedDate: 2008-03-26 14:14:27 +0100 (Mi, 26 Mrz 2008) $
 */

public class Services {
        
    private static LookupCache regCache;
    private static ServiceDiscoveryManager sdm;
    private static LookupDiscoveryManager mgr;
    
    private static Hashtable<Uuid, Starter> remoteProcs;
        
    /*
     * Templates:
     */
    
	private static final ServiceTemplate REG_TMP = new ServiceTemplate(null, new Class[] {Registry.class}, null);
    
    /*
     * Security manager & policy file check:
     */
    
    static {
        if(System.getSecurityManager() == null) {
            System.setSecurityManager(new RMISecurityManager());
        }
        
        try {
        	System.getProperty("java.policy");
        } catch(Exception e) {
        	System.err.println("Security problem: No policy file set.\n");
        	System.exit(1);
        }
    }
    
    /*
     * Logger:
     */
    
    static {
    	
        Logger logger = Logger.getLogger("net.jini.discovery.LookupLocatorDiscovery");
        logger.setLevel(Level.SEVERE);
    	
    }
    
    /*
     * Jini service caching:
     */
    
    private static void init() {
    	if(mgr == null) {
            try {
                mgr = new LookupDiscoveryManager(DiscoveryGroupManagement.ALL_GROUPS, null, null);
                sdm = new ServiceDiscoveryManager(mgr, new LeaseRenewalManager());
                regCache = sdm.createLookupCache(REG_TMP, null, null);
            } catch (Exception e) {
                e.printStackTrace();
                System.exit(1);
            } 
            
            remoteProcs = new Hashtable<Uuid, Starter>();
    	}
    }
    
    /**
     * Finds an arbitrary registry service via unicast or multicast.
     * @param urls URL of the lookup service for unicast localization.
     * @return A registry service or <code>null</code>, if localization failed.
     * @throws MalformedURLException One of the argument URLs is not valid.
     */
    
    public static Registry findRegistry(String... urls) throws MalformedURLException {
        	
    	init();
    	
    	if(urls != null && urls.length > 0) {
    		LookupLocator[] locs = new LookupLocator[urls.length];
    		for (int i = 0; i < locs.length; i++) {
				locs[i] = new LookupLocator(urls[i]);
			}
    		mgr.addLocators(locs);
    	}
    		
    	int tries = 50;
		ServiceItem item = null;
		
		while(item == null && tries-- > 0) {
			item = regCache.lookup(null);
			try {
				Thread.sleep(200);
			} catch (InterruptedException e) {
				// do nothing
			}
		}
		
		if(item != null) {
			return (Registry)item.service;
		}
		
		return null;
    }
    
    /**
     * Finds the processor with the given ID via unicast or multicast.
     * @param pid The processor identifier.
     * @param urls URL of the lookup service for unicast localization.
     * @return The referenced processor or <code>null</code>, if locatozation failed.
     * @throws MalformedURLException One of the argument URLs is not valid.
     */
    
    public static Processor findProcessor(Uuid pid, String... urls) throws MalformedURLException {
    	
    	init();
    	
    	if(urls != null && urls.length > 0) {
    		LookupLocator[] locs = new LookupLocator[urls.length];
    		for (int i = 0; i < locs.length; i++) {
				locs[i] = new LookupLocator(urls[i]);
			}
    		mgr.addLocators(locs);
    	}
    	
    	Entry[] attributes = new Entry[] {new Name(pid.toString())};
        ServiceTemplate tmp = new ServiceTemplate(null, new Class[] {Processor.class}, attributes);
    	
    	try {
     
            ServiceItem item = sdm.lookup(tmp, null, 5000);
            
    	    if (item != null) {
            	return (Processor)item.service;
    	    }
    	    
    	} catch(Exception e) {
    	    // do nothing ...
    	}
    	
    	return null;
    }
    
    /**
     * Starts a new remote processor instance.
     * @param urls URL of the lookup service for unicast localization.
     * @return The proxy of the created processor.
     * @throws UnknownHostException Exception thrown when the machine's host name cannot be localized.
     * @throws MalformedURLException One of the argument URLs is not valid.
     */
    
    public static Processor startProcessor(String... urls) throws UnknownHostException, MalformedURLException {
		Uuid pid = UuidFactory.generate();
		Starter starter = new Starter(pid, urls);
		starter.start();
		remoteProcs.put(pid, starter);
		return findProcessor(pid, urls);
    }
     
    /**
     * Destroys a given processor. This method unexports the processor service
     * and cancels all leases. The service disappears when the leases expire.
     * @param pid The identifier of the processor to destroy.
     * @return <code>true</code>, if the processor has been destroyed successfully.
     */
    
    public static boolean destroyProcessor(Uuid pid) {
    	
    	if(pid == null) {
    		return false;
    	}
    	
        Starter starter = remoteProcs.remove(pid);
        return (starter == null)? false : starter.terminate();
    }
    
    /**
     * Destroys a given processor. This method unexports the processor service
     * and cancels all leases. The service disappears when the leases expire.
     * @param proc The processor to destroy.
     * @return <code>true</code>, if the processor has been destroyed successfully.
     */
    
    public static boolean destroyProcessor(Processor proc) {
        try {
            return (proc != null) && destroyProcessor(proc.getPID());
        } catch (RemoteException e) {
            return false;
        }
    }
    
    /**
     * Destroy all processors that were started from this class.
     * @return <code>true</code>, if all processors have been destroyed successfully.
     */
    
    public static boolean destroyAllProcessors() {
        boolean result = true;
        for(Starter starter : remoteProcs.values()) {
            result &= starter.terminate();
        }
        remoteProcs.clear();
        return result;
    }
}