package nenok.alloc;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
import java.util.Set;

import nenok.lc.Node;
import nenok.net.IdentityLocator;
import nenok.net.Locator;
import nenok.va.Domain;
import nenok.va.Predictor;
import net.jini.id.Uuid;

/**
 * This class performs processor allocation to join tree node by 
 * use of the minimization algorithm for multiway cuts on trees.
 * Basically, it is a static allocation model which means that 
 * processor allocation is fully performed before collect is started.
 * 
 * @author Marc Pouly
 * @version $LastChangedRevision: 553 $<br/>$LastChangedDate: 2008-03-26 14:18:45 +0100 (Mi, 26 Mrz 2008) $
 */

public class Multiway implements Allocator {
    
    private Predictor pred;
    private List<Uuid> procs;
    private Hashtable<Node, Penalty> ndata;
    
    /**
     * Constructor:
     * @param procs The allocatable processors.
     * @param pred The weight predictor for the communication costs.
     * @param root The root node of the jointree.
     * @throws RemoteException Exception caused by accessing the processors that are assigned.
     */
        
    public Multiway(Set<Uuid> procs, Predictor pred, Node root) throws RemoteException {
        procs.remove(null);
        
        this.pred = pred;
        this.procs = new ArrayList(procs);
        this.ndata = new Hashtable<Node, Penalty>(); 
        
        /*
         * Start inward phase:
         */
        
        Set<Node> leaves = root.getLeaves();
        for(Node leaf : leaves) {
            inward(leaf);
        }
        
        /*
         * Allocator root processor:
         */
        
        Penalty penalty = ndata.get(root);
        if(root.getContent() instanceof IdentityLocator) {
        	IdentityLocator id = (IdentityLocator)root.getContent();
        	id.setPID(penalty.getProcessorWithMinPenalty());
        }
        
        /*
         * Start outward phase:
         */
        
        for(Node parent : root.getParents()) {
            outward(parent);
        }
    }
    
    /**
     * @see nenok.alloc.Allocator#allocate(nenok.lc.Node)
     */

    public void allocate(Node node) throws RemoteException {
    	
    	/*
    	 * Processors are already allocated. Only costs remain to be computed.
    	 */
        
    }
        
    /**
     * Inward phase: Computes the penalty values for each node.
     * @param node The current node in the recursive run of the inward phase.
     */
    
    private void inward(Node node) {
        for(Node parent : node.getParents()) {
            if(!ndata.containsKey(parent)) {
                return;
            }
        }
        ndata.put(node, new Penalty(node));
        if(node.getChild() != null) {
            inward(node.getChild());
        }
    }
    
    /**
     * Outward phase: Assigns a processor to each node.
     * @param node The current node in the recursive run of the outward phase.
     * @throws RemoteException Exception caused by accessing the processors that are assigned.
     */
    
    private void outward(Node node) throws RemoteException {
    	
    	/*
    	 * Determine new processor if node contains an identity locator.
    	 */
        
    	Locator loc = (Locator)node.getContent();
        if(loc instanceof IdentityLocator) {
            double min = Double.MAX_VALUE;
            int index = -1;
            for (int i = 0; i < procs.size(); i++) {
                double weight = 0;
                weight += ndata.get(node).valueOf(procs.get(i));
                if(weight < min) {
                    min = weight;
                    index = i;
                }
            }
            ((IdentityLocator)loc).setPID(procs.get(index));
        } 
        
        /*
         * Perform recursion downwards the tree.
         */
        
        for(Node parent : node.getParents()) {
            outward(parent);
        }
    }
    
    /**
     * @see nenok.alloc.Allocator#getName()
     */
    
    public String getName() {
        return "Multiway";
    }
    
    /**
     * Implementation of the penalty function.
     * 
     * @author Marc Pouly
     * @version 1.1
     */
    
    private class Penalty {
                
        /*
         * Penalty function: Processor -> double
         */
        
        private double[] penalties;
        
        /**
         * Constructor:
         * @param node The node whose penalty values are computed.
         */
        
        private Penalty(Node node) {
            
            penalties = new double[procs.size()];
            
            Locator loc = (Locator)node.getContent();
            
            /*
             * Leaves without assigned processor:
             */
            
            if(node.getParents().size() == 0) {
                if(loc instanceof IdentityLocator) {
                    for (int i = 0; i < penalties.length; i++) {
                        penalties[i] = 0;
                    }
                }
            }
            
            /*
             * Nodes with assigned processor:
             */
            
            else if(!(loc instanceof IdentityLocator)) {
                for(int i = 0; i < penalties.length; i++) {
                    if(procs.indexOf(loc.getPID()) == i) {
                        penalties[i] = 0;
                    } else {
                        penalties[i] = Double.MAX_VALUE;
                    }
                }
                return;
            }
            
            /*
             * Intermediate nodes without assigned processor:
             */
            
            else {
                for (int i = 0; i < penalties.length; i++) {
                    double result = 0;
                    for(Node parent : node.getParents()) {
                        Penalty pData = ndata.get(parent);
                        double min = Double.MAX_VALUE;
                        for(int j = 0; j < procs.size(); j++) {
                            double penalty = pData.valueOf(procs.get(j));
                            if(i != j) {
                                penalty += pred.predict(Domain.intersection(parent.getDomain(), node.getDomain()));
                            }
                            if(penalty < min) {
                                min = penalty;
                            }
                        }
                        result += min;
                    }
                    penalties[i] = result;
                }
                return;
            }
        }
        
        /**
         * @param proc A processor
         * @return The penalty if processor <code>proc</code> is assigned to the current node.
         */
        
        private double valueOf(Uuid proc) {
            return penalties[procs.indexOf(proc)];
        }
    
        /**
         * @return The minimum penalty value.
         */
        
        private Uuid getProcessorWithMinPenalty() {
            int index = -1;
            double result = Double.MAX_VALUE;
            for (int i = 0; i < penalties.length; i++) {
                if(penalties[i] < result) {
                    result = penalties[i];
                    index = i;
                }
            }
            return procs.get(index);
        }
    }
}
