package nenok.constr;

import nenok.lc.JoinTree;
import nenok.lc.Node;
import nenok.va.Domain;

/**
 * The join tree construction algorithm base interface. 
 * This is the head class of a realization of the STRATEGY design pattern, 
 * allowing that the same type of join tree architecture can be constructed 
 * with different construction algorithms.
 * 
 * @author Marc Pouly
 * @version $LastChangedRevision: 554 $<br/>$LastChangedDate: 2008-03-26 14:19:57 +0100 (Mi, 26 Mrz 2008) $
 */

public abstract class Algorithm {
    
    private double constrTime = -1;
			
   /**
	* Method to build join trees. The JoinTree.Construction object given as argument
    * contains all necessary informations for the construction process, this is: the knowledgebase,
    * a set of queries that need to be covered by the join tree and the {@link Node} builder method
    * for the type of architecture to be constructed.
	* @param data Object that contains all necessary information to build join trees.
	* @return The root node of the constructed join tree.
	* @throws ConstrException Exceptions caused by the jointree construction process.
    */
	
	public abstract Node buildTree(JoinTree.Construction data) throws ConstrException;

	/**
	 * @return The construction time of the last constructed join tree.
	 */
	
	public double getConstructionTime() {
        return constrTime;
    }
	
    /**
     * Sets the construction time of the last constructed join tree.
     * This method is called by the constructor of {@link JoinTree}.
     * @param time The construction time.
     */
    
    public void setConstructionTime(double time) {
        this.constrTime = time;
    }
    
	/**
	 * @return The name of the construction algorithm.
	 */

	public abstract String getName();
	
	/**
	 * Eliminates node chains within a jointree.
	 * @param node The root node of the subtree that has to be optimized.
	 * @param data The construction data that was used to construct the tree.
	 */
	
	public static void optimize(Node node, JoinTree.Construction data) {
		for(int i = 0; i < node.getParents().size(); i++) {
			Node parent = node.getParents().get(i);
			Domain pDomain = parent.getDomain();
			
			// Ignore full nodes:
			if(parent.isFull()) {
				continue;
			}
						
			// Remove parent if its domain is a subset of the child domain:
			if(pDomain.subSetOf(node.getDomain()) || parent.getParents().size() < 2) {			
				node.getParents().remove(parent);
				parent.setChild(null);
				for(Node pp : parent.getParents()) {
					pp.setChild(node);
					node.addParent(pp);
				}
				parent.getParents().clear();
				if(parent.isFull()) {
					node.setContent(parent.getContent());
				}
				i--;
				continue;
			} 
		}
		
        // Apply this procedure recursively:
        for(Node parent : node.getParents()) {
            optimize(parent, data);
        }
	}
}
