package nenok.lc.hugin;

import nenok.adapt.Adapter;
import nenok.adapt.Labeled;
import nenok.lc.LCException;
import nenok.lc.Node;
import nenok.va.Domain;

/**
 * This is the node class implementation that belongs to the HUGIN architecture.
 * 
 * @author Marc Pouly
 * @version $LastChangedRevision: 557 $<br/>$LastChangedDate: 2008-03-26 14:23:32 +0100 (Mi, 26 Mrz 2008) $
 */

public class HNode extends Node {

    // Internal message cache:
	private int counter;
    private Labeled[] messages;
    
    // Separator:
    private Labeled separator;
    
    /**
     * Constructor:
     * @param adapter The adapter for this node.
     */
    
    public HNode(Adapter adapter) {
        super(adapter);
    }
                
    /**
     * @see nenok.lc.Node#collect()
     */
    
    public void collect() throws LCException { 
        
        // Test, if all parents have sent their message:
        if(counter < this.getParents().size()) {
            return;
        }
                
        // Compute new node content -> messages haven't been initialized for leaves.  
        if(getParents().size() > 0) {
            content = adapter.combine(content, messages);
                    
            // Throw messages away:
            messages = null;
        }
                                   
        // Stop when reaching the root node:
        if(getChild() == null) {
            return;
        }
                      
        // Compute message for the child:
        Domain inter = Domain.intersection(content.label(), getChild().getDomain());  
        separator = adapter.marginalize(content, inter);
                        
        // Send message to child:
        HNode child = (HNode)this.getChild();
        child.receiveParentMessage(separator);
        child.collect();
    }

    /**
     * @see nenok.lc.Node#distribute()
     */
    
    public void distribute() throws LCException {
        for(Node parent : this.getParents()) {
            Labeled message = adapter.marginalize(content, Domain.intersection(this.getDomain(), parent.getDomain()));
        	((HNode)parent).receiveChildMessage(message);
            parent.distribute();
        }
    }
   
    /*
     * Helper methods:
     */
    
    /**
     * Method called by parent node to send message to this node.
     * @param message The message to send.
     */
    
    private void receiveParentMessage(Labeled message) {
        if(messages == null) {
            messages = new Labeled[getParents().size()];
        }
        
    	messages[counter++] = message;
    }
    
    /**
     * Method called by child node to send message to this node.
     * @param message The message to send.
     * @throws LCException Generic local computation exception.
     */
    
    private void receiveChildMessage(Labeled message) throws LCException {
        // Inverse of separator:
        Labeled inverse = adapter.inverse(separator);
                    
        // Combine message with inverse of separator:
        Labeled idempotent = adapter.combine(inverse, message);
        
        // Combine idempotent to the node content:
        content = adapter.combine(idempotent, content);
     }
}
