package nenok.lc.ss;

import java.util.LinkedHashMap;
import java.util.Map;

import nenok.adapt.Adapter;
import nenok.adapt.Labeled;
import nenok.lc.LCException;
import nenok.lc.Node;
import nenok.va.Domain;

/**
 * This is the node class implementation that belongs to the Shenoy-Shafer architecture.
 * Internally, this class stores messages within a {@link LinkedHashMap} in order to ensure
 * that the combination order equals the order messages arrive.
 * 
 * @author Marc Pouly
 * @version $LastChangedRevision: 557 $<br/>$LastChangedDate: 2008-03-26 14:23:32 +0100 (Mi, 26 Mrz 2008) $
 */

public class SSNode extends Node {

    // Internal message cache:
    private LinkedHashMap<Node, Labeled> messages;
        
    /**
     * Constructor:
     * @param adapter The adapter for this node.
     */
    
    public SSNode(Adapter adapter) {
        super(adapter);
        this.messages = new LinkedHashMap<Node, Labeled>(this.getParents().size());
    }
    
    /**
     * @see nenok.lc.Node#collect()
     */
    
    public void collect() throws LCException { 
        
        // Test, if all parents have sent their message:
        if(messages.keySet().size() < this.getParents().size()) {
            return;
        }
        
        // Store initial node content:
        messages.put(this, content);
                
        // Combine messages from parents:
        Labeled result = content;
        for(Map.Entry<Node, Labeled> entry : messages.entrySet()) {
            if(!entry.getKey().equals(this)) {
                result = adapter.combine(result, entry.getValue());  
            }
        }
        content = result;
                          
        // Stop when reaching the root node:
        if(getChild() == null) {
            return;
        }
               
        // Compute message for the child:
        Domain dom = Domain.intersection(content.label(), getChild().getDomain());        
        Labeled message = adapter.marginalize(content, dom);
                        
        // Send message to child:
        SSNode child = (SSNode)this.getChild();
        child.receiveParentMessage(message, this);
        child.collect();
    }

    /**
     * @see nenok.lc.Node#distribute()
     */
    
    public void distribute() throws LCException {
        for(Node parent : this.getParents()) {
            Labeled message = computeParentMessage(parent);
            ((SSNode)parent).receiveChildMessage(message);
            parent.distribute();
        }
        
        // Throw messages away:
        messages = null;
    }
    
    /**
     * Helper method to set the scaling factor to the root node.
     * @param factor The scaling factor.
      * @throws LCException Generic local computation exception.
     */
    
    protected void setScalingFactor(Labeled factor) throws LCException {
        Labeled loc = messages.remove(this);
        Labeled val = adapter.combine(factor, loc);
        messages.put(this, val);
    }
    
    /*
     * Helper methods:
     */
    
    /**
     * Method called by parent node to send message to this node.
     * @param message The message to send.
     * @param parent The addresser node.
     */
    
    private void receiveParentMessage(Labeled message, Node parent){
        messages.put(parent, message);
     }

    /**
     * Method called by child node to send message to this node.
     * @param message The message to send.
     * @throws LCException Generic local computation exception.
     */
    
    private void receiveChildMessage(Labeled message) throws LCException {
        messages.put(this.getChild(), message);
        content = adapter.combine(content, message);
    }
    
    /**
     * Computes the message sent to the given parent.
     * @param parent The parent that will receive the computed message.
     * @return The message for the given parent node.
    */
    
    private Labeled computeParentMessage(Node parent) throws LCException {
        Labeled result = messages.get(this);
        for(Map.Entry<Node, Labeled> entry : messages.entrySet()) {
            if(!entry.getKey().equals(this) && !entry.getKey().equals(parent)) {
                result = adapter.combine(result, entry.getValue());              
            }
        }
        return adapter.marginalize(result, Domain.intersection(result.label(), parent.getDomain()));
    }
}
