package nenok.net;

import java.io.Serializable;
import java.rmi.RemoteException;

import nenok.Services;
import nenok.adapt.Labeled;
import nenok.net.processor.Processor;
import nenok.va.Domain;
import nenok.va.Predictability;
import nenok.va.Predictor;
import nenok.va.Valuation;
import net.jini.id.Uuid;

/**
 * The location of a (remote) {@link Valuation} object can unambiguously be determined by 
 * the identifier of the processor hosting the valuation and the identifier of the valuation's 
 * envelope. Locators are objects that carries these informations in order to localize valuation 
 * objects at a later time. Furthermore, they manage the following additional informations:
 * <br/><br/>
 * <ul>
 * 		<li>The domain of the referenced valuation. Jointree construction algorithms need to know 
 * 			the domains of the valuations to build jointrees without transmitting valuation objects 
 * 			over the network.
 * 		</li>
 * 		<li>The weight of the referenced valuation. It is used as the principal measure for the processor 
 * 			allocation process and needs therefore to be avaiable at this time. 
 * 		</li>  
 * 		<li>Locator objects are collected within knowledgebases. Such knowledgebases should only contain 
 * 			valuations of the same type. Because only references to valuations are stored in knowledgebases, 
 * 			this cannot be garantueed at compile-time. Each locator contains therefore the package and classname 
 * 			(i.e. <code>v.getClass().getName()</code>) for the valuation object it references. Using this information, 
 * 			knowledgebases can identify the valuation types.
 * 		</li>
 * </ul>
 * 
 * @version $LastChangedRevision: 556 $<br/>$LastChangedDate: 2008-03-26 14:21:45 +0100 (Mi, 26 Mrz 2008) $
 */

public class Locator implements Labeled, Serializable {
	
	protected Uuid pid;
	private Uuid envID;
	private Domain dom;
	private String type;
	private int weight;
		
    /**
     * Creator:
     * @param pid The identifier of the processor hosting the valuation.
     * @param env The wrapper envelope of the valuation.
     * @return A new Locator instance corresponding to the valuation stored in the envelope.
     */
    
    public static Locator create(Uuid pid, Envelope env) {
		if(pid == null) {
			throw new IllegalArgumentException("Null is not a valid processor identifier.");
		}
        if(env.getValuation() instanceof Predictability) {
            return new Locator(pid, env).new Predictable(pid, env);
        } 
        return new Locator(pid, env);
    }
	
    /**
     * Constructor:
     */
    
    protected Locator() {}
    
	/**
	 * Constructor:
     * @param pid The identifier of the processor hosting the valuation.
	 */
	
	private Locator(Uuid pid, Envelope env) {
		this.pid = pid;
		this.envID = env.getID();
		this.dom = env.getValuation().label();
        this.weight = env.getValuation().weight();
		this.type = env.getValuation().getClass().getName();
	}
	
    /**
     * @return Retrieves the referenced valuation from the processor.
	 * @throws RemoteException Exception caused when trying to access processors.
     */
	
	public Valuation retrieve() throws RemoteException {
		Processor p = null;
		try {
			p = Services.findProcessor(pid);
		} catch (Exception e) {
			throw new RemoteException("Cannot find processor hosting the valuation.", e);
		}
		return (p != null)? p.get(envID) : null;
	}
	
	/**
	 * Deletes the valuation that is referenced by this locator.
	 * @throws RemoteException Exception caused when trying to access processors.
	 */
	
	public void delete() throws RemoteException {
		Processor p = null;
		try {
			p = Services.findProcessor(pid);
		} catch (Exception e) {
			throw new RemoteException("Cannot find processor hosting the valuation.", e);
		}
		if(p != null) {
			p.delete(this);
		}
	}
	
    /**
     * @return The domain of the referenced valuation.
     */
	
    public Domain label() {
        return dom;
    } 
	
    /**
     * @return The weight of the referenced valuation.
     */

    public int getWeight() {
    	return weight;
    }
	
    /**
     * @return The type (class name) of the referenced valuation.
     */
    
    public String getType() {
    	return type;
    }
    
    /**
     * @return The indentifier of the processor that hosts the valuation.
     */
    
    public Uuid getPID() {
    	return this.pid;
    }
    
	/**
	 * @return The identifier of the envelope that wraps the referenced valuation.
	 */
    
    public Uuid getEnvelopeID() {
    	return envID;
    }
	
    /**
     * @see java.lang.Object#hashCode()
     */

    public int hashCode() {
    	return envID.hashCode();
    }
    
    /**
     * @see java.lang.Object#toString()
     */

    public String toString() {
    	return "(Envelope: "+envID+", Processor: "+pid+")";
    }
    
    /**
     * Two locators are equal, if they point to the same object.
     * @see java.lang.Object#equals(java.lang.Object)
     */

    public boolean equals(Object o) {
    	if(!(o instanceof Locator)) {
    		return false;
    	}
    	Locator loc = (Locator)o;
    	return loc.getPID().equals(getPID()) && loc.envID.equals(envID);
    }

    /**
     * Extension for weight predictable valuation objects.
     * 
     * @author Marc Pouly
     * @version 1.3
     */

     public final class Predictable extends Locator {
         
         private final Predictor predictor;
         
         /**
          * Constructor:
          * @param pid The identifier of the processor hosting the valuation.
          * @param env The wrapper envelope of the valuation.
          * @param host The host running the processor that stores this valuation.
          */
         
         private Predictable(Uuid pid, Envelope env) {
             super(pid, env);
             this.predictor = ((Predictability)env.getValuation()).predictor();
         }
         
         /**
          * @see nenok.net.Locator#getWeight()
          */
         
         public int getWeight() {
             return predictor.predict(dom);
         }
     
         /**
          * @return The weight predictor of the referenced valuation.
          */
         
         public Predictor getPredictor() {
             return predictor;
         }
     }

}
