package nenok.net.processor;

import java.rmi.Remote;
import java.rmi.RemoteException;

import nenok.net.Envelope;
import nenok.net.Locator;
import nenok.va.Domain;
import nenok.va.Valuation;
import net.jini.id.Uuid;

/**
 * The interface of a processor in a Nenok federation.
 * 
 * @author Marc Pouly
 * @version $LastChangedRevision: 556 $<br/>$LastChangedDate: 2008-03-26 14:21:45 +0100 (Mi, 26 Mrz 2008) $
 */

public interface Processor extends Remote {

    /**
     * Stores a valuation wrapped in an envelope on this processor.
     * @param env The envelope with the valuation to store.
     * @return The locator of the stored valuation (envelope).
     * @throws RemoteException Locator methods all throw remote exceptions.
     */
    
    public Locator store(Envelope env) throws RemoteException;
    
    /**
     * Stores a valuation on this processor.
     * @param val The valuation to store.
     * @return The locator of the stored valuation.
     * @throws RemoteException Locator methods all throw remote exceptions.
     */
    
    public Locator store(Valuation val) throws RemoteException;
    
    /**
     * Stores a set of valuations on this processor.
     * @param vals The valuations to store.
     * @return The locators of the stored valuations. Locator i references valuation i.
     * @throws RemoteException Locator methods all throw remote exceptions.
     */
    
    public Locator[] store(Valuation[] vals) throws RemoteException;
        
    /**
     * Retrieves a stored valuation from this processor.
     * This method call does not remove the corresponding locator from the internal store.
     * @param envID The identifier of the valuation's envelope.
     * @return The valuation referenced by the locator or null.
     * @throws RemoteException Locator methods all throw remote exceptions.
     */
    
    public Valuation get(Uuid envID) throws RemoteException; 
    
    /**
     * Is the valuation with the given envelope ID stored on this processor.
     * @param envID The valuation's envelope ID.
     * @return <code>true</code>, if the valuation exists.
     * @throws RemoteException Locator methods all throw remote exceptions.
     */
    
    public boolean exists(Uuid envID) throws RemoteException;
    
    /**
     * Deletes a stored valuation from this processor.
     * @param loc The locator referencing the valuation to delete.
     * @throws RemoteException Locator methods all throw remote exceptions.
     */
    
    public void delete(Locator loc) throws RemoteException;
    
    /**
     * Deletes all stored valuations.
     * @throws RemoteException Locator methods all throw remote exceptions.
     */
    
    public void reset() throws RemoteException; 
    
    /*
     * Computation:
     */
    
    /**
     * Combines the valuations specified by locators and stores the result.
     * @param loc A locator object.
     * @param locs Further locators that have to be combined.
     * @return The locator that points to the combined valuation.
     * @throws RemoteException Locator methods all throw remote exceptions.
     */
    
    public Locator combine(Locator loc, Locator... locs) throws RemoteException;
    
    /**
     * Marginalizes the valuations specified by the locator to be given domain.
     * @param loc The locator pointing to the valuation to be marginalized.
     * @param dom The domain onto which the marginalization is performed.
     * @return The locator that points to the marginalized valuation.
     * @throws RemoteException Locator methods all throw remote exceptions.
     */
    
    public Locator marginalize(Locator loc, Domain dom) throws RemoteException;
    
    /**
     * Inverses the valuations specified by the locator.
     * @param loc The locator pointing to the valuation to be inverted.
     * @return The locator that points to the inverse valuation.
     * @throws RemoteException Locator methods all throw remote exceptions.
     */
    
    public Locator inverse(Locator loc) throws RemoteException;    
    
    /*
     * Informational methods:
     */
        
    /**
     * @return The identifier of this processor.
     * @throws RemoteException Locator methods all throw remote exceptions.
     */
    
    public Uuid getPID() throws RemoteException;

    /**
     * @return The codebase of this processor at start time.
     * @throws RemoteException Locator methods all throw remote exceptions.
     */
    
    public String getCodebase() throws RemoteException;
}