package nenok.net.processor;

import java.rmi.RemoteException;
import java.util.concurrent.ConcurrentHashMap;
import java.util.logging.Level;
import java.util.logging.Logger;

import nenok.net.Envelope;
import nenok.net.Locator;
import nenok.va.Domain;
import nenok.va.Separativity;
import nenok.va.Valuation;
import net.jini.id.Uuid;

/**
 * An implementation of the {@link Processor} interface.
 * 
 * @author Marc Pouly
 * @version $LastChangedRevision: 556 $<br/>$LastChangedDate: 2008-03-26 14:21:45 +0100 (Mi, 26 Mrz 2008) $
 */

public class ProcessorImpl implements Processor {
    
    private final Uuid pid;
    private String codebase;
    private final ConcurrentHashMap<Uuid, Envelope> table;
    
    /*
     * Ensure that a SecurityManager is running.
     */
    
    private static Logger logger;
    
    static {
        logger = Logger.getLogger(Processor.class.getPackage().toString());
        logger.setLevel(Level.WARNING);
    }
                
    /**
     * Constructor:
     * @param pid The identifier of the processor.
     */
    
    public ProcessorImpl(Uuid pid) {
        this.pid = pid;
        this.table = new ConcurrentHashMap<Uuid, Envelope>(); 
    	try {
    		this.codebase = System.getProperty("java.rmi.server.codebase");
    	} catch(Exception e) {
    		System.err.println("Exception from Security-Manager: Probably no policy file defined.");
    		e.printStackTrace();
    		System.exit(1);
    	}
    }
    
    /**
     * @see nenok.net.processor.Processor#store(nenok.va.Valuation)
     */
    
    public Locator store(Valuation val) throws RemoteException {
        Envelope env = new Envelope(val);
        table.put(env.getID(), env);
        return Locator.create(pid, env);
    }
    
    /**
     * @see nenok.net.processor.Processor#store(nenok.net.Envelope)
     */
    
    public Locator store(Envelope env) throws RemoteException {
        table.put(env.getID(), env);
        return Locator.create(pid, env);
    }

    /**
     * @see nenok.net.processor.Processor#store(nenok.va.Valuation[])
     */
    
    public Locator[] store(Valuation[] vals) throws RemoteException {
        Locator[] result = new Locator[vals.length];
        for (int i = 0; i < vals.length; i++) {
            result[i] = this.store(vals[i]);
        }
        return result;
    }

	/**
	 * @see nenok.net.processor.Processor#get(net.jini.id.Uuid)
	 */

    public Valuation get(Uuid envID) throws RemoteException {
    	if(envID == null) {
    		throw new RemoteException("Cannot retrieve valuation with identifier null.");
    	}
    	Envelope env = table.get(envID);
        return (env != null) ? env.getValuation() : null;
    }

    /**
     * @see nenok.net.processor.Processor#exists(net.jini.id.Uuid)
     */
    
    public boolean exists(Uuid envID) throws RemoteException {
        return table.containsKey(envID);
    }

    /**
     * @see nenok.net.processor.Processor#delete(nenok.net.Locator)
     */
    
    public void delete(Locator loc) throws RemoteException {
        table.remove(loc.getEnvelopeID()); 
    }

    /**
     * @see nenok.net.processor.Processor#reset()
     */
    
    public void reset() throws RemoteException {
        table.clear();
    }
    
    /**
     * @see nenok.net.processor.Processor#getPID()
     */
    
    public Uuid getPID() throws RemoteException {
        return this.pid;
    }

    /**
     * @see nenok.net.processor.Processor#getCodebase()
     */

    public String getCodebase() throws RemoteException {
        return this.codebase;
    }

    /**
     * @see nenok.net.processor.Processor#combine(nenok.net.Locator, nenok.net.Locator[])
     */
    
    public Locator combine(Locator loc, Locator... locs) throws RemoteException {
    	Valuation result = loc.retrieve();
    	for (int i = 0; i < locs.length; i++) {
			result = result.combine(locs[i].retrieve());
		}
    	return store(result);
    }
    
    /**
     * @see nenok.net.processor.Processor#marginalize(nenok.net.Locator, nenok.va.Domain)
     */
    
    public Locator marginalize(Locator loc, Domain dom) throws RemoteException {
    	Valuation result = loc.retrieve();
    	return store(result.marginalize(dom)); 	
    }
    
    /**
     * @see nenok.net.processor.Processor#inverse(nenok.net.Locator)
     */
    
    public Locator inverse(Locator loc) throws RemoteException {
    	if(!loc.getPID().equals(pid)) {
    		throw new RemoteException("Unable to find valuation referenced by the locator "+loc+" on this processor.");
    	}
    	Separativity result = (Separativity)loc.retrieve();
    	return store(result.inverse());
    }
    
    /**
     * @see java.lang.Object#equals(java.lang.Object)
     */
    
    public boolean equals(Object o) {
        if(o instanceof Processor) {
            try {
                return ((Processor)o).getPID().equals(this.getPID());
            } catch (RemoteException e) {
                return false;
            }
        }
        return false;
    }
    
    /**
     * @see java.lang.Object#toString()
     */
    
    public String toString() {
        return "Processor "+pid;
    }
}
