package nenok.net.processor;

import java.io.IOException;
import java.net.UnknownHostException;
import java.rmi.RMISecurityManager;
import java.rmi.Remote;
import java.rmi.server.ExportException;
import java.util.logging.Level;
import java.util.logging.Logger;

import net.jini.core.discovery.LookupLocator;
import net.jini.core.entry.Entry;
import net.jini.core.lookup.ServiceID;
import net.jini.discovery.DiscoveryGroupManagement;
import net.jini.discovery.LookupDiscoveryManager;
import net.jini.export.Exporter;
import net.jini.id.Uuid;
import net.jini.id.UuidFactory;
import net.jini.jeri.BasicILFactory;
import net.jini.jeri.BasicJeriExporter;
import net.jini.jeri.tcp.TcpServerEndpoint;
import net.jini.lease.LeaseRenewalManager;
import net.jini.lookup.JoinManager;
import net.jini.lookup.entry.Name;
import net.jini.lookup.entry.ServiceInfo;

import com.sun.jini.config.ConfigUtil;
import com.sun.jini.lookup.entry.BasicServiceType;

/**
 * This class launches Nenok Processor instances and registers them in a JINI lookup service.
 * 
 * @author Marc Pouly
 * @version $LastChangedRevision: 556 $<br/>$LastChangedDate: 2008-03-26 14:21:45 +0100 (Mi, 26 Mrz 2008) $
 */

public class Starter extends Thread {
    
    private Uuid pid;
    private String[] urls;
    private Exporter exporter;  
    private JoinManager joinMgr;
    
    /*
     * Implementation note: Do not delete this reference.
     * the garbage collector would delete the reference.
     */
    
    private String ip;
    private ProcessorImpl impl;
    private boolean loop = true;
        
    /*
     * Ensure that a SecurityManager is running.
     */
    
    private static Logger logger;
    
    static {
        if(System.getSecurityManager() == null) {
            System.setSecurityManager(new RMISecurityManager());
        }
        logger = Logger.getLogger(Processor.class.getPackage().toString());
        logger.setLevel(Level.WARNING);        
    }
        
    /**
     * Constructor:
     * @param pid The identifier of this processor.
     * @param urls URL of the lookup service for unicast localization.
     * @throws UnknownHostException Exception thrown if the host name of the current machine cannot be found.
     */
    
    public Starter(Uuid pid, String... urls) throws UnknownHostException {
        this.pid = pid;
        this.urls = urls;
        ip = ConfigUtil.getHostName();
    }
    
    /**
     * @see java.lang.Thread#run()
     */
    
    public void run() {
        
        /*
         * Service attributes:
         * Implementation note: Name entry is used for service lookup. Don't change it !
         */
            
        Entry[] attributes = new Entry[] {new Name(pid.toString()), 
                                          new ServiceInfo("Nenok Processor", "Nenok Developer", "", "1.0", "", ""),
                                          new BasicServiceType("Nenok Processor")};
                
        Uuid uuid = UuidFactory.generate();
        ServiceID id = new ServiceID(uuid.getMostSignificantBits(), uuid.getLeastSignificantBits());     
                        
        /*
         * Create proxy instance.
         */
        
        impl = new ProcessorImpl(pid);
        exporter = new BasicJeriExporter(TcpServerEndpoint.getInstance(ip, 0), new BasicILFactory());


        Remote proxy = null;
        try {
            proxy = exporter.export(impl);
        } catch (ExportException e) {
            logger.severe("Unable to create proxy.");
            e.printStackTrace();
            System.exit(2);
        }
        
        if(proxy == null) {
            logger.severe("Unable to create proxy.");
            System.exit(3);            
        }
                
        /*
         * Register proxy:
         */
        
        try {
            LookupDiscoveryManager mgr = new LookupDiscoveryManager(DiscoveryGroupManagement.ALL_GROUPS, null, null);
            
        	if(urls != null && urls.length > 0) {
        		LookupLocator[] locs = new LookupLocator[urls.length];
        		for (int i = 0; i < locs.length; i++) {
    				locs[i] = new LookupLocator(urls[i]);
    			}
        		mgr.addLocators(locs);
        	}
            
            joinMgr = new JoinManager(proxy, attributes, id, mgr, new LeaseRenewalManager());
        } catch(IOException e) {
            e.printStackTrace();
            System.exit(4);
        }
                  
        if(joinMgr == null) {
            logger.severe("Unable to create join manager.");
            System.exit(5); 
        }
                
        while(joinMgr.getJoinSet() == null || joinMgr.getJoinSet().length == 0) {
            try {
                Thread.sleep(100);
            } catch (InterruptedException e) {}
        }
                        
        logger.info("Service successfully registered ...");
        
        /*
         * Loop:
         */
        
        while(loop) {
            try {
                Thread.sleep(100);
            } catch (InterruptedException e) {}
        }
        
        logger.info("Service stopped ...");
     }
    
    /**
     * Terminates this service. This method unexports the processor service
     * and cancels all leases. The service disappears when the leases expire.
     * @return <code>true</code>, if the service has been unexported sucessfully. 
     */
    
    public boolean terminate() {
        this.loop = false;
        joinMgr.getLeaseRenewalManager().clear();
        return exporter.unexport(true);
    }

    /**
     * @return The IP of the machine running the created processor instance.
     */
    
    public String getIP() {
        return ip;
    }
}