package nenok.net.registry;

import java.rmi.RemoteException;
import java.util.Arrays;
import java.util.Collection;
import java.util.Hashtable;
import java.util.logging.Level;
import java.util.logging.Logger;

import nenok.Knowledgebase;
import nenok.net.Locator;

/**
 * A possible implementation of the NENOK knowledgebase registry server with an internal 
 * {@link Hashtable}. This associative storage links names of knowledgebases together with 
 * {@link Locator} objects that belong to this knowledgebase.
 * 
 * @author Marc Pouly
 * @version $LastChangedRevision: 556 $<br/>$LastChangedDate: 2008-03-26 14:21:45 +0100 (Mi, 26 Mrz 2008) $
 */

public class RegistryImpl implements Registry {
	
    private String ip;
	private Hashtable<String, RemoteKB> table;
	
	/*
	 * Logging framework:
	 */
	
	private static final Logger logger; 
	static {
		logger = Logger.getLogger(Registry.class.getPackage().toString());
		logger.setLevel(Level.INFO);
	}
	
	/**
	 * Constructor:
     * @param host The name of the host running this registry.
	 */
	
	public RegistryImpl(String host) {
		this.ip = host;
        this.table = new Hashtable<String, RemoteKB>();	
	}
	
	/*
	 * Registry interface:
	 */
	
	/**
	 * @see nenok.net.registry.Registry#createKnowledgebase(java.lang.String)
	 */
	
	public boolean createKnowledgebase(String name) {
		if(table.containsKey(name)) {
			logger.info("Knowledgebase "+name+" exists already.");
			return false;
		}
		RemoteKB kb = new RemoteKB(name);
		table.put(name, kb);
		logger.finest("Knowledgebase "+name+" has been created.");
		return true;
	}
		
	/**
	 * @see nenok.net.registry.Registry#deleteKnowledgebase(java.lang.String)
	 */
	
	public boolean deleteKnowledgebase(String knowledgebase) {
		if((table.remove(knowledgebase)) != null) {
			logger.finest("Knowledgebase "+knowledgebase+" has been deleted.");
			return true;
		}
		logger.info("Knowledgebase "+knowledgebase+" does not exist.");
		return false;
	}

	/**
	 * @see nenok.net.registry.Registry#getKnowledgebase(java.lang.String)
	 */
    	
	public Knowledgebase getKnowledgebase(String name) {
		return (name == null)? null : table.get(name);
	}
	
	/**
     * @see nenok.net.registry.Registry#getKnowledgebases()
     */
    
    public String[] getKnowledgebases() throws RemoteException {
        String[] result = new String[table.size()];
        int i = 0;
        for(String key : table.keySet()) {
            result[i++] = key;
        }
        return result;
    }

    /**
	 * @see nenok.net.registry.Registry#add(java.lang.String, nenok.net.Locator)
	 */
	
	public boolean add(String knowledgebase, Locator locator) throws RemoteException {
		if(!table.containsKey(knowledgebase)) {
			logger.info("Knowledgebase "+knowledgebase+" does not exist.");
			return false;
		}
        RemoteKB kb = table.get(knowledgebase);
        return kb.add(locator);
	}

	/**
	 * @see nenok.net.registry.Registry#addAll(java.lang.String, java.util.Collection)
	 */
	
	public boolean addAll(String knowledgebase, Collection<Locator> locators) throws RemoteException {
        if(!table.containsKey(knowledgebase)) {
			logger.info("Knowledgebase "+knowledgebase+" does not exist.");
			return false;
		}
        RemoteKB kb = table.get(knowledgebase);
        return kb.addAll(locators);
	}

    /**
     * @see nenok.net.registry.Registry#addAll(java.lang.String, nenok.net.Locator[])
     */
    
    public boolean addAll(String knowledgebase, Locator[] locators) throws RemoteException {
        return addAll(knowledgebase, Arrays.asList(locators));
    }
    
	/**
	 * @see nenok.net.registry.Registry#remove(java.lang.String, nenok.net.Locator)
	 */
	
	public boolean remove(String knowledgebase, Locator locator) throws RemoteException {
		RemoteKB kb = table.get(knowledgebase);
		if(kb != null) {
			return kb.remove(locator);
		} 
        logger.info("Knowledgebase "+knowledgebase+" does not exist.");
        return false;
	}

	/**
	 * @see nenok.net.registry.Registry#contains(java.lang.String, nenok.net.Locator)
	 */
	    
	public boolean contains(String knowledgebase, Locator locator) throws RemoteException {
		RemoteKB kb = table.get(knowledgebase);
		if(kb != null) {
			return kb.contains(locator);
		}
		logger.info("Knowledgebase "+knowledgebase+" does not exist.");
		return false;
	}

	/**
	 * @see nenok.net.registry.Registry#exists(java.lang.String)
	 */
	
	public boolean exists(String knowledgebase) throws RemoteException {
		return (table.get(knowledgebase)) != null;
	}
	
	/**
	 * @see nenok.net.registry.Registry#reset()
	 */
	
	public void reset() throws RemoteException {
		table.clear();
		logger.finest("Registry service has been reseted.");	
	}

    /**
     * @see nenok.net.registry.Registry#getIP()
     */
    
    public String getIP() throws RemoteException {
        return this.ip;
    }
}
