package nenok.net.registry;

import java.io.Serializable;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;

import nenok.Knowledgebase;
import nenok.Services;
import nenok.Utilities;
import nenok.net.Locator;
import nenok.net.processor.Processor;
import nenok.va.Domain;
import nenok.va.Identity;
import nenok.va.Predictability;
import nenok.va.Predictor;
import nenok.va.Valuation;

/**
 * A knowledgebase is basically a set of {@link Locator} objects. Because each locator identifies unambigously a valuation 
 * object stored on a Nenok processor, this represents indeed the mathematical idea of a distributed knowledgebase.
 * <br/><br/>
 * Knowledgebases contain only elements of the same type. However, because {@link Locator} objects are only pointer on the 
 * storage place (processor) of a valuation, this type checking cannot be done at compile-time. So, the first added element 
 * determines which type this knowledgebase accepts. If this constraint is not satisfied, an {@link IllegalArgumentException} is thrown.
 * 
 * @author Marc Pouly
 * @version $LastChangedRevision: 556 $<br/>$LastChangedDate: 2008-03-26 14:21:45 +0100 (Mi, 26 Mrz 2008) $
 */

public class RemoteKB extends Knowledgebase implements Serializable, Iterable<Locator> {
    
    private String type;
    private HashSet<Locator> locators;
	private volatile Class cache;
    
    /**
     * Constructor:
     * @param name The knowledgebase's name.
     */
    
    protected RemoteKB(String name) {
    	super(name);
        locators = new HashSet<Locator>();
    }
        
    /**
     * Constructor:
     * @param name The knowledgebase's name.
     * @param locators The locators to add to the new knowledgebase.
     */
        
    public RemoteKB(Locator[] locators, String name) {
        this(name);
        addAll(Arrays.asList(locators));
    }
            
    /**
     * Adds a new locator to this knowledgebase.
     * @param loc The locator to add.
     * @return <code>true</code>, it the locator has been added successfully.
     */
    
    public boolean add(Locator loc) {
        if(loc.getType().equals(Identity.class.getName())) {
            return locators.add(loc);
        }
        if(type == null || type == "") {
            type = loc.getType();
            return locators.add(loc);
        }
        if(this.type.equals(loc.getType())) {
            return locators.add(loc);
        }
        return false;
    }
    
    /**
     * Adds a new collection of locators to this knowledgebase.
     * @param locators The locators to add.
     * @return <code>true</code>, it all locators have been added successfully.
     */
    
    public boolean addAll(Collection<Locator> locators) {
        boolean result = true;
        for(Locator l : locators) {
            result &= this.add(l);
        }
        return result;
    }
    
    /**
     * Adds a new array of locators to this knowledgebase.
     * @param locators The locators to add.
     * @return <code>true</code>, it all locators have been added successfully.
     */
    
    public boolean addAll(Locator[] locators) {
        boolean result = true;
        for(Locator l : locators) {
            result &= this.add(l);
        }
        return result;
    }    
    
    /**
     * Removes a given locator from this knowledgebase.
     * @param loc The locator to remove.
     * @return <code>true</code>, if the locator has been removed successfully.
     */
    
    public boolean remove(Locator loc) {
        return locators.remove(loc);
    }
    
    /**
     * Looks for a given locator within this knowledgebase.
     * @param loc The locator to search.
     * @return <code>true</code>, if the locator has been found.
     */
    
    public boolean contains(Locator loc) {
        return locators.contains(loc);
    }
        
    /**
     * @see nenok.Knowledgebase#size()
     */
    
    @Override
    public int size() {
    	return locators.size();
    }
    
    /**
     * @see nenok.Knowledgebase#toArray()
     */
    
    @Override
    public Locator[] toArray() {
    	return locators.toArray(new Locator[locators.size()]);
    }
    
    /**
     * @see nenok.Knowledgebase#getValuations()
     */
    
    @Override
    public Valuation[] getValuations() throws RemoteException {
    	Valuation[] result = new Valuation[locators.size()];
    	int i = 0;
    	for(Locator loc : locators) {
    		result[i++] = loc.retrieve();
    	}
    	return result;
    }
    
    /**
     * @return The predictor of this knowledgebase of null, if it doesn't exist.
     * @throws RemoteException This operation performs remote calls and therefore forewards RemoteExceptions.
     */
    
    public Predictor getPredictor() throws RemoteException {
    	if(!Utilities.interfaceOf(Predictability.class, this.getType())) {
    		return null;
    	}
    	for(Locator loc : locators) {
	        if(loc instanceof Locator.Predictable) {
	        	return ((Locator.Predictable)loc).getPredictor();
	        }
	    }
	    return null;
    }
    
    /*
     * Iterable interface:
     */
    
    /**
     * @return The iterator for this knowledgebase.
     * @see java.lang.Iterable#iterator()
     */
    
    public Iterator<Locator> iterator() {
        return locators.iterator();
    }
    
    /*
     * Misc:
     */
    
    /**
     * @see nenok.Knowledgebase#getDomain()
     */
    
    @Override
    public Domain getDomain() {
    	ArrayList<Domain> doms = new ArrayList<Domain>(locators.size());
    	for(Locator loc : locators) {
			doms.add(loc.label());
		}
    	return Domain.union(doms);
    }
     
    /**
     * Loads dynamically the class type of the valuations referenced by this knowledgebase.
     * Returns <code>null</code>, if this knowledgebase is empty.
     * @see nenok.Knowledgebase#getType()
     */
    
    @Override
    public Class getType() throws RemoteException {
    	if(cache != null) {
    		return cache;
    	}
    	reload();
    	return cache;
    }
    
    /**
     * Loads dynamically the class type of the valuations referenced by this knowledgebase.
     * @throws RemoteException This operation performs remote calls and therefore forewards RemoteExceptions.
     */
    
    private void reload() throws RemoteException {
        if(type != null && type != "") {
            for(Locator loc : locators) {
                if(this.type.equals(loc.getType())) {
                    try {
                    	
                		Processor p = null;
                		try {
                			p = Services.findProcessor(loc.getPID());
                		} catch (Exception e) {
                			throw new RemoteException("Cannot find processor hosting the valuation.", e);
                		}
                		
                        if(p == null) {
                        	throw new RemoteException("Processor of locator "+loc+" cannot be accessed.");
                        }
                        
                        /*
                         * For local processors:
                         */
                                               
                        if(p.getCodebase() == null) {
                        	
                        	throw new RemoteException("No codebase set for this application.");
                        	
                        	// With system class loader:
                        	//ClassLoader loader = ClassLoader.getSystemClassLoader();
                        	//cache = loader.loadClass(type);
                        	//return;
                        	
                        	/*if(cache != null) {
                        		return;
                        	}
                        	
                        	// With JAR loader:
                        	cache = Utilities.loadInstance(type);
                        	return;*/
                        
                        }
                                                
                        /*
                         * For remote processors:
                         */

                        String[] str = p.getCodebase().split(" ");                        
                        URL[] codebase = new URL[str.length];
                        for (int i = 0; i < codebase.length; i++) {
                            codebase[i] = new URL(str[i]);
                        }    
                        URLClassLoader loader = new URLClassLoader(codebase);
                        cache =  loader.loadClass(type);
                        return;
                    } catch (ClassNotFoundException e) {
                        throw new RemoteException("Unable to locate valuation's class from processor codebase.", e);
                    } catch (MalformedURLException e) {
                        throw new RemoteException("Malformed URL received from processor.", e);
                    }
                }
            }            
        }
        cache = Identity.class;
    }
}

