package nenok.va;

import nenok.va.sr.SRValuation;
import nenok.va.sr.SeparativeSemiring;

/**
 * This interface equips a valuation algebra with some concept of division.
 * NENOK does not distinguish between separative and regular valuation algebras,
 * both are represented by this interface. The division itself is realized by a 
 * single method that computes the inverse element of the current valuation.
 * 
 * @author Marc Pouly
 * @version $LastChangedRevision: 559 $<br/>$LastChangedDate: 2008-03-26 14:25:18 +0100 (Mi, 26 Mrz 2008) $
 */

public interface Separativity extends Valuation {
	
	/**
	 * Returns an inverse element for the current valuation.
	 * @return The inverse valuation of the given domain.
	 */
	
	public Separativity inverse();
	
	/**
	 * Implementor class of this interface for separative semiring valuations.
	 * 
	 * @author Marc Pouly
	 * @version 1.4
	 */
	
	public class Implementor {
		
		// Singleton design pattern:
		private static Implementor implementor;
		
		/**
		 * Constructor:
		 */
		
		private Implementor() {}
		
		/**
		 * @return An instance of this implementor class.
		 */
		
		public static Implementor getInstance() {
			if(implementor == null) {
				implementor = new Implementor();
			}
			return implementor;
		}
		
		/**
		 * Computes the inverse element of a separative semiring valuation.
		 * @param val The separative semiring valuation.
		 * @return The inverse of the argument valuation.
		 */
		
		public Separativity inverse(SRValuation val) {
	    	SeparativeSemiring[] inverses = new SeparativeSemiring[val.getValues().length];
	    	for (int i = 0; i < inverses.length; i++) {
				inverses[i] = ((SeparativeSemiring)(val.getValues()[i])).inverse();
			}
	    	return (Separativity)val.create(val.getVariables(), inverses);
		}
	}
}

