package nenok.va.sr;

import java.util.Arrays;

import nenok.va.Domain;
import nenok.va.FiniteVariable;
import nenok.va.Identity;
import nenok.va.Predictability;
import nenok.va.Predictor;
import nenok.va.Representor;
import nenok.va.VAException;
import nenok.va.Valuation;
import nenok.va.Variable;

/**
 * Specifies a semiring induced valuation by leading its principal operations of 
 * marginalization &amp; combination back to the according semiring 
 * operations.
 * 
 * @author Marc Pouly
 * @version $LastChangedRevision: 560 $<br/>$LastChangedDate: 2008-03-26 14:37:05 +0100 (Mi, 26 Mrz 2008) $
 */

public abstract class SRValuation implements Valuation, Predictability {
    
    protected FiniteVariable[] variables;
    protected Semiring[] values;
    private Domain domain;
    
    // SINGLETON design pattern.
    private static SRPredictor self;
    
    /**
     * Factory method:
	 * @param vars The variables of this semiring valuation. 
	 * @param values The semiring value for each possible configuration.
	 * @return A new instance of this class.
     */
    
    public abstract SRValuation create(FiniteVariable[] vars, Semiring[] values);
        
	/**
	 * Constructor:
	 * @param vars The variables of this semiring valuation. 
	 * @param values The semiring value for each possible configuration.<br/> 
	 * The number of values must match with the product computed from each 
	 * variable's frame cardinality.
	 */
		
	public SRValuation(FiniteVariable[] vars, Semiring[] values) {
		
		/*
		 * Check user input:
		 */
		
		if(vars == null || values == null) {
	        throw new IllegalArgumentException("Invalid arguments: At least one argument is null.");
	    }
	    if(FiniteVariable.countConfigurations(vars) != values.length) {
	        throw new IllegalArgumentException("Dimension of semiring value array doesn't " +
	        		"match. Given: "+values.length+", expected: "+FiniteVariable.countConfigurations(vars)+".");
	    }	
	    
	    /*
	     * Sort variable array:
	     */
	    
	    
		FiniteVariable[] copy = new FiniteVariable[vars.length];
		for (int i = 0; i < vars.length; i++) {
			copy[i] = vars[i];
		}	    
	    
	    Arrays.sort(copy);
	    this.variables = copy;
	    
	    /*
	     * Adapt values to new configuration arrangement.
	     */
	    
	    if(Arrays.equals(vars, copy)) {
	    	this.values = values;
	    } else {
	    	Semiring[] newValues = new Semiring[values.length];
	    	for (int i = 0; i < values.length; i++) {
				int index = Configuration.convert(i, vars, copy);
				newValues[index] = values[i];
			}
	    	this.values = newValues;
	    } 
	}
	
    /*
     * Valuation implementation:
     */
    
    /**
     * @see nenok.va.Valuation#label()
     */
	
    public Domain label() {
    	if(domain == null) {
    		domain = new Domain(variables);
    	}
        return domain;
    }
    
    /**
     * @see nenok.va.Valuation#combine(nenok.va.Valuation)
     */
     
    public Valuation combine(Valuation val) {
		if(val instanceof Identity) {
			return this;
		}
        SRValuation arg = (SRValuation)val;
        //Number of variable in the new domain:
        int inter = 0;
        for (int i = 0; i < variables.length; i++) {
			if(Arrays.binarySearch(arg.variables, variables[i]) >= 0) {
				inter++;
			}
		}
		//Variable array of the new potential:
		FiniteVariable[] newVars = new FiniteVariable[arg.variables.length + this.variables.length - inter];
		Variable[] interVars = new Variable[inter];
        
		//Fill new variable array and compute dimension of probability array:
		int elementSize = 1, newVarsCounter = 0, interCounter = 0;
		for (; newVarsCounter < this.variables.length; newVarsCounter++) {
			newVars[newVarsCounter] = this.variables[newVarsCounter];
			elementSize *= this.variables[newVarsCounter].getFrame().length;
		}
		for (int j = 0; j < arg.variables.length; j++) {
			if(Arrays.binarySearch(variables, arg.variables[j]) < 0) {
				newVars[newVarsCounter++] = arg.variables[j];
				elementSize *= arg.variables[j].getFrame().length;
			} else {
				interVars[interCounter++]= arg.variables[j];
			}
		}
		
		//Arithmetic array of the new potential:
		Semiring[] newValues = new Semiring[elementSize];
		
		/*
		 * Compute each configuration's value
		 */
			
		int[] symbols = new int[inter];
		int thisConfgurations = FiniteVariable.countConfigurations(this.variables);
		int argConfigurations = FiniteVariable.countConfigurations(arg.variables);
		
		// Determine indexes of intervars:
		int thisIndex[] = new int[symbols.length];
		int argIndex[] = new int[symbols.length];
		for (int i = 0; i < interVars.length; i++) {
			thisIndex[i] = Arrays.binarySearch(variables, interVars[i]);
			argIndex[i] = Arrays.binarySearch(arg.variables, interVars[i]);
		}
		
		for (int i = 0, configurationIndex = 0; i < thisConfgurations; i++) {
		    //Store all variables' indexes of this.variables:
		    for (int j = 0; j < interVars.length; j++) {
		    	symbols[j] = Configuration.getSymbolIndex(variables, thisIndex[j], i);
            }
		    for (int j = 0; j < argConfigurations; j++) {
		        boolean multiply = true;
		        for (int k = 0; k < interVars.length; k++) {
                    multiply &= (Configuration.getSymbolIndex(arg.variables, argIndex[k], j) == symbols[k]);
                }
		        //Perform multiplication:
		        if(multiply) {
		            newValues[configurationIndex++] = this.values[i].multiply(arg.values[j]);
		        }
            }
        }
	
		return create(newVars, newValues);
    }
    
    /**
     * @see nenok.va.Valuation#marginalize(nenok.va.Domain)
     */
      
	public Valuation marginalize(Domain dom) {
		if(dom.equals(label())) {
		    return this;
		}
		if(!dom.subSetOf(label())) {
			new VAException("Marginalization impossible: this.label() = "+label()+", argument = "+dom+".");
		}
		//Calculates the difference set between both domains.
		Domain d = Domain.difference(label(), dom);
		FiniteVariable[] elim = d.toArray(FiniteVariable.class);
		
		// Eliminate each variable in the difference set.
		
		FiniteVariable[] vars = variables;
		Semiring[] elements = values;
		
		for (int n = 0; n < elim.length; n++) {
			
			FiniteVariable var = elim[n];
			FiniteVariable[] newVars = new FiniteVariable[vars.length-1];
	        int oldPosition = -1;
	        int size = 1;
	        
	        // Fill the new variable array and calculate the number of configurations of the new potential.
	        for (int i = 0, j = 0; i < vars.length; i++) {
	            if(!vars[i].equals(var)) {
	                size *= vars[i].getFrame().length;
	                newVars[j++] = vars[i];
	            } else {
	                oldPosition = i;
	            }
	        }
	        
	        //The new element array.
	        Semiring[] newElements = new Semiring[size];
	        int offset = 1;
	        for (int i = oldPosition + 1; i < vars.length; i++) {
	            offset *= vars[i].getFrame().length;
	        }
	        
	        // Calculate the elements of the probability & extension array.
	        int blocs = (elements.length / var.getFrame().length) / offset;
	        int blocLines = newElements.length / blocs;
	        for (int i = 0, index = 0; i < blocs; i++) {
	            int base = i * (elements.length / blocs);
	            for (int j = 0; j < blocLines; j++) {
	                Semiring sum = null; 
	                for (int k = 0; k < var.getFrame().length; k++) {   
	                	int position = base + j + k*offset;
	                	if(sum == null) {
	                		sum = elements[position];
	                	} else {
	                		sum = sum.add(elements[position]);
	                	}
	                }
	                newElements[index] = sum;
	                index++;
	            }
	        }
	        
	        vars = newVars;
	        elements = newElements;
		}
		
		return create(vars, elements);
	}

    /**
     * @see nenok.va.Valuation#weight()
     */
    
    public int weight() {
        return Predictability.Implementor.getInstance().weight(this);
    }
	
    /*
     * Predictablility:
     */
    
    /**
     * @see nenok.va.Predictability#predictor()
     */
    
    public Predictor predictor() {
        if(self == null) {
            self = new SRPredictor();
        }
        return self;
    }
	
	/*
	 * Equality check:
	 */
	
    /**
     * @see java.lang.Object#equals(java.lang.Object)
     */
    
	public boolean equals(Object o) {
		if(!(o instanceof SRValuation)) {
			return false;
		}
		SRValuation p = (SRValuation)o;
		return Arrays.equals(p.variables, variables) && Arrays.equals(p.values, values);
	}
    
    /**
     * Returns the semiring value of the given configuration. In order to make this
     * method independent from the variable (configuration) order, the variable array 
     * to which the configuration is relativ is given.
     * @param vars The variable array that specifies the configuration's order.
     * @param conf The configuration whose value is returned.
     * @return The value of the given configuration or <code>null</code> if no such 
     * configuration exists.
     */
    
	public Semiring evaluate(FiniteVariable[] vars, Object[] conf) {
		if(vars.length != conf.length) {
			return null;
		}
		int result = Configuration.encode(conf, vars);
		result = Configuration.convert(result, vars, variables);
		return values[result];
	}
		    
	/**
     * @see java.lang.Object#toString()
     */
	
	@Representor
    public String toString() {
        int max = 0;
        String format = "", result = "", line = "", title = "Value";
        
        /*
         * Find string of maximum length:
         */
        
        for (int i = 0; i < variables.length; i++) {
            FiniteVariable v = variables[i];
            if(max < v.toString().length()) {
                max = v.toString().length();
            }
            for (int j = 0; j < v.getFrame().length; j++) {
                if(max < v.getFrame()[j].toString().length()) {
                    max = v.getFrame()[j].toString().length();
                }
            }
        }
        
        if(max < title.length()) {
            max = title.length();
        }
        
        max += 2;
        
        /*
         * Prepare format string for title:
         */

        for (int i = 0; i < variables.length+1; i++) {
        	format += "|%"+(i+1)+"$-"+max+"s";
        }
        format += "|\n";
        
        /*
         * Output header line:
         */
        
        Object[] data = new Object[variables.length+1];
        for (int i = 0; i < variables.length; i++) {
            data[i] = variables[i];
        }
        data[variables.length] = title;
        result += String.format(format, data);
        
        /*
         * Horizontal line:
         */
        
        for (int i = 0; i < result.length()-1; i++) {
            line += "-";
        }
        line += "\n";
        result = line+result+line;

        /*
         * Prepare format string for configurations:
         */

        format = "";
        for (int i = 0; i < variables.length; i++) {
        	format += "|%"+(i+1)+"$-"+max+"s";
        }
        format += "|%"+(variables.length+1)+"$-"+max+"s|\n";
                
        /*
         * Configurations:
         */
        
        for (int i = 0; i < values.length; i++) {
            Object row[] = new Object[data.length];
            for (int j = 0; j < variables.length; j++) {
                row[j] = variables[j].getFrame()[Configuration.getSymbolIndex(variables, j,i)].toString();
            }
            row[data.length-1] = values[i];
            result += String.format(format, row);
        }   
        
        result += line;
		return result;
    }

    /*
     * Getters:
     */
    
    /**
     * @return The array of semiring values.
     */
    
	public Semiring[] getValues() {
		return values;
	}

    /**
     * @return The variable array of this semiring valuation.
     */

	public FiniteVariable[] getVariables() {
		return variables;
	}

	/*
	 * Helper methods:
	 */
		
	/**
	 * Implementation of weight predictor class.
	 * 
	 * @author Marc Pouly
	 * @version 1.4
	 */
	
	public class SRPredictor implements Predictor {
	    	    	    
	    /**
	     * @see nenok.va.Predictor#predict(nenok.va.Domain)
	     */
	    
	    public int predict(Domain dom) {
	        if(dom.size() == 0) {
	            return 0;
	        }
	        int result = 1;
	        for(Variable var : dom) {
	            FiniteVariable v = (FiniteVariable)var;
	            result *= v.getFrame().length;
	        }
	        return result;
	    }
	}
}